<?php

/**
 * Define the plugin settings
 *
 * @link       https://signatu.com
 * @since      1.0.0
 *
 * @package    Signatu
 * @subpackage Signatu/includes
 */

/**
 * Define the Signatu API Proxy.
 *
 *
 * @since      1.0.0
 * @package    Signatu
 * @subpackage Signatu/includes
 * @author     Signatu AS <wordpress@signatu.com>
 */


class Signatu_API {

	private $consent_api = 'https://api.signatu.com/consent/v1';
	private $dataprocessing_api = 'https://api.signatu.com/dataprocessing/v0';
	private $trackerdetect_api = 'https://api.signatu.com/trackerdetect/v0';

	public function authorize(WP_REST_Request $request) {
		/** 
		 * We need to allow anonymous access to site. However, we want to limit requests from outside of the site.
		 * We use nonces to limit the extent of this, although it's not going to eliminate the risk.
		 */
		$nonce = $request->get_header( 'X-WP-Nonce');
		return wp_verify_nonce($nonce, 'wp_rest');	
	}

	public function register_api() {
		/** 
		 * @see https://developer.wordpress.org/rest-api/extending-the-rest-api/adding-custom-endpoints/ 
		 * @see https://developer.wordpress.org/reference/functions/register_rest_route/
		 * */
		$ns = 'signatu/consent/v1';
		register_rest_route( $ns, '/(?P<path>.+)', array(
			'methods' => array('GET', 'POST'),
			'callback' => array($this, 'handle_consent_request'),
			'permission_callback' => array($this, 'authorize'),
		  ) );
		
		$ns = 'signatu/dataprocessing/v0';
		register_rest_route( $ns, '/(?P<path>.+)', array(
			'methods' => array('GET', 'PATCH', 'POST'),
			'callback' => array($this, 'handle_dataprocessing_request'),
			'permission_callback' => array($this, 'authorize'),
		) );
		$ns = 'signatu/trackerdetect/v0';
		register_rest_route( $ns, '/(?P<path>.+)', array(
			'methods' => array('GET'),
			'callback' => array($this, 'handle_trackerdetect_request'),
			'permission_callback' => array($this, 'authorize'), // '__return_true',
		) );
	}

	/** @todo forward the headers here as well - required for ETags etc to work */
	private function handle_response($res) {
		/** $headers is a case-insensitive map of the values */
		$headers = wp_remote_retrieve_headers($res);

		$status = $res['response']['code'];
		/** End debug */
		if(!is_wp_error($res) && $status < 400 ) {
			$body = json_decode($res['body']);
			/** @see https://developer.wordpress.org/reference/classes/wp_rest_response/ */
			$response = new WP_REST_Response( $body );
			
			// Add a custom status code
			$response->set_status( $status );

			$forward_headers = array(
				'etag' => $headers['etag'],
			);
			$response->set_headers($forward_headers);

			
			return $response;
		}
		/** Error */
		$body = json_decode($res['body']);
		$code = $res['response']['message'];
		return new WP_Error( $code, $body->message, array( 'status' => $res['response']['code'] ) );
	}


	public function proxy_request(WP_REST_Request $request, $baseUrl) {
		/** @see https://developer.wordpress.org/reference/classes/wp_rest_request/  */
		$json = $request->get_json_params();
		$path = wp_unslash($request->get_param('path'));
		$url = $baseUrl . '/' . $path;

		$access_token = get_option('signatu_config_access_token');

		$req_headers = $request->get_headers();
	
		$headers = array(
		);

		/** For some reason PHP figures the dashes should be made _. Go figure */
		$forward_headers = array('if-none-match');
		foreach($req_headers as $key => $header) {
			$lkey = str_replace('_', '-', $key);
			if (in_array(strtolower($lkey), $forward_headers)) {
				$headers[$lkey] = implode(', ', $header);
			}
		}

		$headers['Authorization'] = 'Bearer ' . $access_token;
		$headers['Content-Type'] = 'application/json';

		
		/** @see https://developer.wordpress.org/reference/classes/WP_Http/request/ for args */
		$args = array(
			'method' => $request->get_method(),
			'headers' => $headers,
			'body' => json_encode($json),
			'timeout' => 30, 
			'blocking' => true
		);
		$res = wp_remote_request( $url , $args);
		
		return $this->handle_response($res);
	}

	public function handle_consent_request(WP_REST_Request $request) {
		return $this->proxy_request($request, $this->consent_api);
	}

	public function handle_dataprocessing_request(WP_REST_Request $request) {
		return $this->proxy_request($request, $this->dataprocessing_api);
	}

	public function handle_trackerdetect_request(WP_REST_Request $request) {
		return $this->proxy_request($request, $this->trackerdetect_api);
	}
}


