<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://signatu.com
 * @since      1.0.0
 *
 * @package    Signatu
 * @subpackage Signatu/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Signatu
 * @subpackage Signatu/public
 * @author     Signatu AS <wordpress@signatu.com>
 */
class Signatu_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Signatu_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Signatu_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/signatu-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Signatu_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Signatu_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		/** 
		 * Cookie Button. 
		 * @todo support the banner without the button too
		 */
		$base_url = plugin_dir_url( __FILE__ ) . 'js/packaged/' . $this->version;

		//  @see https://wordpress.stackexchange.com/questions/93732/using-get-option-in-javascript
		$handle = 'signatu-script';
		$script = plugin_dir_url( __FILE__ ) . 'js/signatu-public.js';
		wp_register_script( $handle, $script, array( 'jquery' ), $this->version, false );


		$current_user = get_current_user_id();
		$site_url = get_site_url();

		/** 
		 * @see https://developer.wordpress.org/rest-api/using-the-rest-api/authentication/ 
		 * 
		 * Note - the nonce must be named wp_rest to allow access to the API when logged in
		 * */
		$nonce = wp_create_nonce('wp_rest');
		
		$scriptData = array(
				/** Authenticate the client with the REST API. We want to accept only traffic and messages originating from the WP Site */
				'origin' => get_site_url(),
				'app' => get_option('signatu_config_app_type'),
				'nonce' => $nonce,
				'baseUrl' => $base_url,
				'version' => $this->version,
				'endPoints' => array(
					'consentAPI' => $site_url . '/?rest_route=/signatu/consent/v1',
					'dataprocessingAPI' => $site_url . '/?rest_route=/signatu/dataprocessing/v0',
					'trackerdetectAPI' => $site_url . '/?rest_route=/signatu/trackerdetect/v0',
				), 
				
				'language' => get_option('signatu_config_language'),
				'groupId' => get_option( 'signatu_config_group_id' ), 
				'position' => get_option( 'signatu_config_position' ), 
				'variant' => get_option( 'signatu_config_variant' ), 
				'subject' => $current_user != 0 ? $current_user : NULL,
				'privacyPolicyUrl' => get_option('signatu_config_privacy_policy')
			);
					
			wp_localize_script($handle, 'signatuConfig', $scriptData);
			
			wp_enqueue_script($handle);
		}

}
